package com.ruoyi.service.impl;

import com.ruoyi.common.constant.HttpStatus;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.domain.Task;
import com.ruoyi.service.CarReviewTaskService;
import com.ruoyi.service.SystemReviewTaskService;
import com.ruoyi.service.TaskService;
import com.ruoyi.service.TaskState;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Objects;

@Transactional
@Service
public class TaskStateNew implements TaskState, InitializingBean {

    @Autowired
    private TaskStateContext taskStateContext;

    @Autowired
    private ApplicationContext applicationContext;

    @Autowired
    private TaskService taskService;

    @Autowired
    private SystemReviewTaskService systemReviewTaskService;

    @Autowired
    private CarReviewTaskService carReviewTaskService;

    @Override
    public void doCreate(Task task, List<String> taskList) {

        String nextNode = null;

        if(taskList.contains(Task.SUB_TASK_TRFIS)) {
            nextNode = Task.SUB_TASK_TRFIS;
        }
        if(taskList.contains(Task.SUB_TASK_GRFIS)) {
            // 保存车型审查问卷任务
            nextNode = Task.SUB_TASK_GRFIS;;
        }
        if(taskList.contains(Task.SUB_TASK_SRAIF)) {
            // 保存体系审查任务
            nextNode = Task.SUB_TASK_SRAIF;
        }

        task.setNextNode(nextNode);

        taskService.updateById(task);
    }

    @Override
    public void doStart(Task task) {

        task.setTaskStatus(Task.TASK_STATUS_PENDING);
        task.setRunStatus(Task.RUN_STATUS_STATUS_RUNNING);

        if (Objects.equals(task.getNextNode(), Task.SUB_TASK_SRAIF)) {
            systemReviewTaskService.start(task.getSystemReviewTaskId());
            task.setNextNode(Task.SUB_TASK_GRFIS);
            taskService.updateById(task);

        } else if (Objects.equals(task.getNextNode(), Task.SUB_TASK_GRFIS)) {
            carReviewTaskService.startReviewTask(task.getCarReviewTaskId());
            task.setNextNode(Task.SUB_TASK_TRFIS);
            taskService.updateById(task);

        }

        taskService.updateById(task);
    }

    @Override
    public void doSubmit(Task task) {
        throw new ServiceException("不能提交一个尚未开始的任务", HttpStatus.ERROR);
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        taskStateContext.putResource(Task.TASK_STATUS_NEW, applicationContext.getBean(this.getClass()));
    }
}
