package com.ruoyi.service.impl;

import com.ruoyi.common.constant.HttpStatus;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.domain.CarReviewTask;
import com.ruoyi.domain.SystemReviewTask;
import com.ruoyi.domain.Task;
import com.ruoyi.mapper.ReviewDetailsMapper;
import com.ruoyi.service.CarReviewTaskService;
import com.ruoyi.service.SystemReviewTaskService;
import com.ruoyi.service.TaskService;
import com.ruoyi.service.TaskState;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Objects;

@Transactional
@Service
public class TaskStatePending implements TaskState, InitializingBean {

    @Autowired
    private TaskStateContext taskStateContext;

    private ReviewDetailsMapper reviewDetailsMapper;

    @Autowired
    private TaskService taskService;

    @Autowired
    private ApplicationContext applicationContext;

    @Autowired
    private SystemReviewTaskService systemReviewTaskService;

    @Autowired
    private CarReviewTaskService carReviewTaskService;

    @Override
    public void doCreate(Task task, List<String> taskList) {
        throw new ServiceException("不能创建一个已经开始的任务", HttpStatus.ERROR);
    }

    @Override
    public void doStart(Task task) {

        task.setTaskStatus(Task.TASK_STATUS_PENDING);
        task.setRunStatus(Task.RUN_STATUS_STATUS_RUNNING);

        if (Objects.equals(task.getNextNode(), Task.SUB_TASK_SRAIF)) {
            systemReviewTaskService.start(task.getSystemReviewTaskId());
            task.setNextNode(Task.SUB_TASK_GRFIS);
            taskService.updateById(task);

        } else if (Objects.equals(task.getNextNode(), Task.SUB_TASK_GRFIS)) {
            carReviewTaskService.startReviewTask(task.getCarReviewTaskId());
            task.setNextNode(Task.SUB_TASK_TRFIS);
            taskService.updateById(task);

        }

    }

    @Override
    public void doSubmit(Task task) {

        long subTaskCount = 0;

        if(task.getSystemReviewTaskId() != null) {
            subTaskCount = reviewDetailsMapper.count(task.getSystemReviewTaskId(), SystemReviewTask.TYPE);
        }
        if(task.getCarReviewTaskId() != null) {
            subTaskCount = reviewDetailsMapper.count(task.getCarReviewTaskId(), CarReviewTask.TYPE);
        }

        if(subTaskCount > 0) {
            task.setTaskStatus(Task.TASK_STATUS_PENDING);
            task.setRunStatus(Task.RUN_STATUS_RUNNABLE);
        } else {
            task.setTaskStatus(Task.TASK_STATUS_FINISH);
            task.setRunStatus(Task.RUN_STATUS_STATUS_RUNNING);
        }

        taskService.updateById(task);
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        taskStateContext.putResource(Task.TASK_STATUS_PENDING, applicationContext.getBean(this.getClass()));
    }
}
